@@ -1,11 +1,10 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2005  PowerDNS.COM BV
+    Copyright (C) 2002 - 2005  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    it under the terms of the GNU General Public License version 2 as 
+    published by the Free Software Foundation
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -101,7 +100,7 @@ void ZoneParser::parse(const string &fname, const string &origin, unsigned int d
       }
       if(eatLine(line,rec))
 	for(vector<Record>::const_iterator i=rec.begin();i!=rec.end();++i)
-	  d_callback(domain_id,i->name, i->qtype,i->content,i->ttl,i->prio);
+	  d_callback(domain_id, i->name, i->qtype, i->content, i->ttl, i->prio);
     }
 
     //    if(ferror(fds.top())) {
@@ -182,6 +181,8 @@ void ZoneParser::fillRec(const string &qname, const string &qtype, const string
   rec.content=content;
   rec.ttl=ttl;
   rec.prio=prio;
+
+  //  cerr<<"filled with type: "<<rec.qtype<<", "<<QType::chartocode(qtype.c_str())<<": "<<content<<endl;
   recs.push_back(rec);
 }
 
@@ -233,7 +234,7 @@ bool ZoneParser::eatLine(const string& line, vector<Record> &rec)
     tline=lastfirstword+"\t"+tline;
 
   vector<string> parts;
-  stringtok(parts,tline," \t\"");  // THIS IS WRONG, THE " SHOULD BE TREATED! XXX FIXME
+  stringtok(parts,tline," \t");  // we used to strip " here
   if(parts[0][0]!='$' && !isspace(parts[0][0]))
     lastfirstword=parts[0];
 
@@ -531,18 +532,18 @@ bool ZoneParser::parseLine(const vector<string>&words, vector<Record>&rec)
       qname+="."+d_origin;
 
 
-//  cerr<<qname<<", "<<qclass<<", "<<qtype<<", "<<ttl<<", rest from field "<<cpos<<endl;
+  //  cerr<<qname<<", "<<qclass<<", "<<qtype<<", "<<ttl<<", rest from field "<<cpos<<endl;
 	  
   int left=words.size()-cpos;
   string content;
 
-   if((qtype=="MX" && left==2) || (qtype=="SRV" && left==4)){
-     int prio=atoi(words[cpos++].c_str());left--;
-     content=words[cpos++];left--;
-     
-     while(left--)
-       content+=" "+words[cpos++];
-     
+  if((qtype=="MX" && left==2) || (qtype=="SRV" && left==4)){
+    int prio=atoi(words[cpos++].c_str());left--;
+    content=words[cpos++];left--;
+    
+    while(left--)
+      content+=" "+words[cpos++];
+    
     if(content=="@")
       content=d_origin;
     else
@@ -567,7 +568,13 @@ bool ZoneParser::parseLine(const vector<string>&words, vector<Record>&rec)
     }
     if(qtype=="SOA")
       soaCanonic(content);
-    
+    if(qtype=="TXT" && content.size() > 2) {  // strip quotes from TXT
+      if(content[0]=='"')
+	content=content.substr(1);
+      if(content[content.size()-1]=='"')
+	content.resize(content.size()-1);
+    }
+      
     fillRec(qname, qtype, content,ttl, 0, rec);
     return true;
   }
