@@ -30,6 +30,9 @@
 #include "arguments.hh"
 #include "session.hh"
 #include "packetcache.hh"
+#include <boost/lexical_cast.hpp>
+
+using namespace boost;
 
 void CommunicatorClass::addSuckRequest(const string &domain, const string &master, bool priority)
 {
@@ -86,6 +89,9 @@ void CommunicatorClass::suck(const string &domain,const string &remote)
 	  return;
 	}
 	i->domain_id=domain_id;
+	if(i->qtype.getCode()>=1024)
+	  throw DBException("Database can't store unknown record type "+lexical_cast<string>(i->qtype.getCode()-1024));
+
 	di.backend->feedRecord(*i);
       }
     }
@@ -317,8 +323,13 @@ int CommunicatorClass::doNotifications()
   bool purged;
   while(d_nq.getOne(domain, ip, &id, purged)) {
     if(!purged) {
-      d_nresolver.notify(d_nsock,domain,ip,id);
-      drillHole(domain,ip);
+      try {
+	d_nresolver.notify(d_nsock, domain, ip, id);
+	drillHole(domain, ip);
+      }
+      catch(ResolverException &re) {
+	L<<Logger::Error<<"Error trying to resolve '"+ip+"' for notifying '"+domain+"' to server: "+re.reason<<endl;
+      }
     }
     else
       L<<Logger::Error<<Logger::NTLog<<"Notification for "<<domain<<" to "<<ip<<" failed after retries"<<endl;
@@ -393,7 +404,7 @@ void CommunicatorClass::makeNotifySocket()
 
 void CommunicatorClass::notify(const string &domain, const string &ip)
 {
-  d_nq.add(domain,ip);
+  d_nq.add(domain, ip);
 
   d_any_sem.post();
 }
