@@ -433,9 +433,9 @@ void DNSPacket::fillSOAData(const string &content, SOAData &data)
   // fill out data with some plausible defaults:
   // 10800 3600 604800 3600
   data.serial=0;
-  data.refresh=10800;
-  data.retry=3600;
-  data.expire=604800;
+  data.refresh=arg().asNum("soa-refresh-default");
+  data.retry=arg().asNum("soa-retry-default");
+  data.expire=arg().asNum("soa-expire-default");
   data.default_ttl=arg().asNum("soa-minimum-ttl");
 
   vector<string>parts;
@@ -544,7 +544,7 @@ void DNSPacket::addSOARecord(const string &domain, const string & content, uint3
   
   uint32_t *i_p=(uint32_t *)piece5;
   
-  uint32_t soaoffset;
+  uint32_t soaoffset=0;
   if(soadata.serial && (soaoffset=arg().asNum("soa-serial-offset")))
     if(soadata.serial<soaoffset)
       soadata.serial+=soaoffset; // thank you DENIC
@@ -613,7 +613,7 @@ void DNSPacket::addRPRecord(const string &domain, const string &content, uint32_
  string piece1;
 
  toqname(domain.c_str(),&piece1);
- char p[10];
+ char p[11];
  makeHeader(p,17,ttl);
  
  // content contains: mailbox-name more-info-domain (Separated by a space)
@@ -670,7 +670,7 @@ void DNSPacket::addNAPTRRecord(const string &domain, const string &content, uint
 
   //xtoqname(domain.c_str(),&piece1);
   toqname(domain.c_str(),&piece1);
-  char p[10];
+  char p[11];
   makeHeader(p,QType::NAPTR,ttl);
  
   // content contains: 100  100  "s"   "http+I2R"   ""    _http._tcp.foo.com.
@@ -991,28 +991,7 @@ void DNSPacket::wrapup(void)
   stable_sort(rrs.begin(),rrs.end(),rrcomp);
 
   if(!d_tcp && !arg().mustDo("no-shuffle")) {
-    // now shuffle! start out with the ANSWER records  
-    vector<DNSResourceRecord>::iterator first, second;
-    for(first=rrs.begin();first!=rrs.end();++first) 
-      if(first->d_place==DNSResourceRecord::ANSWER && first->qtype.getCode() != QType::CNAME) // CNAME must come first
-	break;
-    for(second=first;second!=rrs.end();++second)
-      if(second->d_place!=DNSResourceRecord::ANSWER)
-	break;
-    
-    if(second-first>1)
-      random_shuffle(first,second);
-    
-    // now shuffle the additional records
-    for(first=second;first!=rrs.end();++first) 
-      if(first->d_place==DNSResourceRecord::ADDITIONAL && first->qtype.getCode() != QType::CNAME) // CNAME must come first
-	break;
-    for(second=first;second!=rrs.end();++second)
-      if(second->d_place!=DNSResourceRecord::ADDITIONAL)
-	break;
-    
-    if(second-first>1)
-      random_shuffle(first,second);
+    shuffle(rrs);
   }
   d_wrapped=true;
 
