@@ -203,6 +203,11 @@ string DLNotifyHostHandler(const vector<string>&parts, Utility::pid_t ppid)
   ostringstream os;
   if(parts.size()!=3)
     return "syntax: notify-host domain ip";
+
+  struct in_addr inp;
+  if(!Utility::inet_aton(parts[2].c_str(),&inp))
+    return "Unable to convert '"+parts[2]+"' to an IP address";
+
   L<<Logger::Warning<<"Notification request to host "<<parts[2]<<" for domain '"<<parts[1]<<"' received"<<endl;
   Communicator.notify(parts[1],parts[2]);
   return "Added to queue";
