@@ -36,6 +36,8 @@
 #include "arguments.hh"
 #include "sstuff.hh"
 #include "syncres.hh"
+#include "dnswriter.hh"
+#include "dnsparser.hh"
 
 LWRes::LWRes()
 {
@@ -57,11 +59,11 @@ LWRes::~LWRes()
 /** Never throws! */
 int LWRes::asyncresolve(const string &ip, const char *domain, int type, bool doTCP)
 {
-  DNSPacket p;
-  p.setQuestion(Opcode::Query,domain,type);
-  p.setRD(false);
-  p.wrapup();
+  vector<uint8_t> vpacket;
+  DNSPacketWriter pw(vpacket, domain, type);
 
+  pw.getHeader()->rd=0;
+  pw.getHeader()->id=random();
   d_domain=domain;
   d_type=type;
   d_inaxfr=false;
@@ -82,25 +84,24 @@ int LWRes::asyncresolve(const string &ip, const char *domain, int type, bool doT
   dt.set();
 
   if(!doTCP) {
-    if(asendto(p.getData(), p.len, 0, (struct sockaddr*)(&toaddr), sizeof(toaddr),p.d.id)<0) {
+    if(asendto((const char*)&*vpacket.begin(), vpacket.size(), 0, (struct sockaddr*)(&toaddr), sizeof(toaddr), pw.getHeader()->id)<0) {
       return -1;
     }
   
     // sleep until we see an answer to this, interface to mtasker
     
-    ret=arecvfrom(reinterpret_cast<char *>(d_buf), d_bufsize-1,0,(struct sockaddr*)(&toaddr), &addrlen, &d_len, p.d.id);
+    ret=arecvfrom(reinterpret_cast<char *>(d_buf), d_bufsize-1,0,(struct sockaddr*)(&toaddr), &addrlen, &d_len, pw.getHeader()->id);
   }
   else {
-    //    cerr<<"do tcp"<<endl;
     Socket s(InterNetwork, Stream);
     IPEndpoint ie(ip, 53);
     s.setNonBlocking();
     s.connect(ie);
 
-    unsigned int len=htons(p.len);
+    unsigned int len=htons(vpacket.size());
     char *lenP=(char*)&len;
-    const char *msgP=p.getData();
-    string packet=string(lenP, lenP+2)+string(msgP, msgP+p.len);
+    const char *msgP=(const char*)&*vpacket.begin();
+    string packet=string(lenP, lenP+2)+string(msgP, msgP+vpacket.size());
 
     if(asendtcp(packet, &s) == 0) {
       //      cerr<<"asendtcp: timeout"<<endl;
@@ -122,7 +123,7 @@ int LWRes::asyncresolve(const string &ip, const char *domain, int type, bool doT
       //      cerr<<"arecvtcp: timeout"<<endl;
       return 0;
     }
-    if(len > d_bufsize) {
+    if(len > (unsigned int)d_bufsize) {
       d_bufsize=len;
       delete[] d_buf;
       d_buf = new unsigned char[d_bufsize];
@@ -139,15 +140,30 @@ int LWRes::asyncresolve(const string &ip, const char *domain, int type, bool doT
 
 LWRes::res_t LWRes::result()
 {
-  DNSPacket p;
-
   try {
-    if(p.parse((char *)d_buf, d_len)<0)
-      throw LWResException("resolver: unable to parse packet of "+itoa(d_len)+" bytes");
-    d_aabit=p.d.aa;
-    d_tcbit=p.d.tc;
-    d_rcode=p.d.rcode;
-    return p.getAnswers();
+    MOADNSParser mdp((const char*)d_buf, d_len);
+    //    if(p.parse((char *)d_buf, d_len)<0)
+    //      throw LWResException("resolver: unable to parse packet of "+itoa(d_len)+" bytes");
+    d_aabit=mdp.d_header.aa;
+    d_tcbit=mdp.d_header.tc;
+    d_rcode=mdp.d_header.rcode;
+
+    LWRes::res_t ret;
+    for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {          
+      //      cout<<i->first.d_place<<"\t"<<i->first.d_label<<"\tIN\t"<<DNSRecordContent::NumberToType(i->first.d_type);
+      //      cout<<"\t"<<i->first.d_ttl<<"\t"<< i->first.d_content->getZoneRepresentation()<<endl;
+      DNSResourceRecord rr;
+      rr.qtype=i->first.d_type;
+      rr.qname=i->first.d_label;
+      rr.ttl=i->first.d_ttl;
+      rr.content=i->first.d_content->getZoneRepresentation();  // this should be the serialised form
+      
+      rr.d_place=(DNSResourceRecord::Place) i->first.d_place;
+      ret.push_back(rr);
+    }
+
+    return ret;
+    //    return p.getAnswers();
   }
   catch(...) {
     d_rcode=RCode::ServFail;
