@@ -106,16 +106,36 @@ bool chopOff(string &domain)
   return true;
 }
 
+bool ciEqual(const string& a, const string& b)
+{
+  if(a.size()!=b.size())
+    return false;
+
+  string::size_type pos=0, epos=a.size();
+  for(;pos < epos; ++pos)
+    if(dns_tolower(a[pos])!=dns_tolower(b[pos]))
+      return false;
+  return true;
+}
+
 /** does domain end on suffix? Is smart about "wwwds9a.nl" "ds9a.nl" not matching */
 bool endsOn(const string &domain, const string &suffix) 
 {
-  if(toLower(domain)==toLower(suffix) || suffix.empty())
+  if( suffix.empty() || ciEqual(domain, suffix) )
     return true;
   if(domain.size()<=suffix.size())
     return false;
-  return (toLower(domain.substr(domain.size()-suffix.size()-1,suffix.size()+1))=="."+toLower(suffix));
-}
+  
+  string::size_type dpos=domain.size()-suffix.size()-1, spos=0;
+  if(domain[dpos++]!='.')
+    return false;
 
+  for(; dpos < domain.size(); ++dpos, ++spos)
+    if(dns_tolower(domain[dpos]) != dns_tolower(suffix[spos]))
+      return false;
+
+  return true;
+}
 
 int sendData(const char *buffer, int replen, int outsock)
 {
@@ -136,7 +156,6 @@ int sendData(const char *buffer, int replen, int outsock)
   return 0;
 }
 
-
 void parseService(const string &descr, ServiceTuple &st)
 {
 
@@ -331,3 +350,31 @@ string makeHexDump(const string& str)
 }
 
 
+
+// shuffle, maintaining some semblance of order
+void shuffle(vector<DNSResourceRecord>& rrs)
+{
+  vector<DNSResourceRecord>::iterator first, second;
+  for(first=rrs.begin();first!=rrs.end();++first) 
+    if(first->d_place==DNSResourceRecord::ANSWER && first->qtype.getCode() != QType::CNAME) // CNAME must come first
+      break;
+  for(second=first;second!=rrs.end();++second)
+    if(second->d_place!=DNSResourceRecord::ANSWER)
+      break;
+  
+  if(second-first>1)
+    random_shuffle(first,second);
+  
+  // now shuffle the additional records
+  for(first=second;first!=rrs.end();++first) 
+    if(first->d_place==DNSResourceRecord::ADDITIONAL && first->qtype.getCode() != QType::CNAME) // CNAME must come first
+      break;
+  for(second=first;second!=rrs.end();++second)
+    if(second->d_place!=DNSResourceRecord::ADDITIONAL)
+      break;
+  
+  if(second-first>1)
+    random_shuffle(first,second);
+
+  // we don't shuffle the rest
+}
