@@ -186,7 +186,7 @@ int PacketHandler::doDNSCheckRequest(DNSPacket *p, DNSPacket *r, string &target)
   DNSResourceRecord rr;
 
   if (p->qclass == 3 && p->qtype.getName() == "HINFO") {
-    rr.content = "PowerDNS $Id: packethandler.cc 518 2005-10-16 15:36:51Z ahu $";
+    rr.content = "PowerDNS $Id: packethandler.cc 539 2005-11-11 11:17:47Z ahu $";
     rr.ttl = 5;
     rr.qname=target;
     rr.qtype=13; // hinfo
@@ -206,7 +206,7 @@ int PacketHandler::doVersionRequest(DNSPacket *p, DNSPacket *r, string &target)
   const string mode=arg()["version-string"];
   if(p->qtype.getCode()==QType::TXT && target=="version.bind") {// TXT
     if(mode.empty() || mode=="full") 
-      rr.content="Served by POWERDNS "VERSION" $Id: packethandler.cc 518 2005-10-16 15:36:51Z ahu $";
+      rr.content="Served by POWERDNS "VERSION" $Id: packethandler.cc 539 2005-11-11 11:17:47Z ahu $";
     else if(mode=="anonymous") {
       r->setRcode(RCode::ServFail);
       return 1;
@@ -389,7 +389,8 @@ int PacketHandler::makeCanonic(DNSPacket *p, DNSPacket *r, string &target)
     int hits=0;
 
     while(B.get(rr)) {
-      hits++;
+      if(rr.qtype.getCode()!=QType::NS || p->qtype.getCode()==QType::NS)
+	hits++;
       if(!rfound && rr.qtype.getCode()==QType::CNAME) {
 	found=true;
 	r->addRecord(rr);
@@ -540,7 +541,7 @@ bool validDNSName(const string &name)
     if(!((c >= 'a' && c <= 'z') ||
 	 (c >= 'A' && c <= 'Z') ||
 	 (c >= '0' && c <= '9') ||
-	 c =='-' || c == '_' || c=='*' || c=='.'))
+	 c =='-' || c == '_' || c=='*' || c=='.' || c=='/'))
       return false;
   }
   return true;
@@ -636,6 +637,7 @@ DNSPacket *PacketHandler::question(DNSPacket *p)
     mret=makeCanonic(p, r, target); // traverse CNAME chain until we have a useful record (may actually give the correct answer!)
 
     if(mret==2) { // there is some data, but not of the correct type
+      DLOG(L<<"There is some data, but not of the correct type"<<endl);
       SOAData sd;
       if(getAuth(p, &sd, target, 0)) {
 	rr.qname=sd.qname;
