@@ -36,7 +36,9 @@
 
 extern MemRecursorCache RC;
 
-map<string,NegCacheEntry> SyncRes::s_negcache;    
+SyncRes::negcache_t SyncRes::s_negcache;    
+SyncRes::nsspeeds_t SyncRes::s_nsSpeeds;    
+
 unsigned int SyncRes::s_queries;
 unsigned int SyncRes::s_outgoingtimeouts;
 unsigned int SyncRes::s_outqueries;
@@ -98,14 +100,20 @@ int SyncRes::doResolve(const string &qname, const QType &qtype, vector<DNSResour
   return res<0 ? RCode::ServFail : res;
 }
 
-string SyncRes::getA(const string &qname, int depth, set<GetBestNSAnswer>& beenthere)
+vector<string> SyncRes::getAs(const string &qname, int depth, set<GetBestNSAnswer>& beenthere)
 {
-  vector<DNSResourceRecord> res;
-  string ret;
+  typedef vector<DNSResourceRecord> res_t;
+  res_t res;
 
-  if(!doResolve(qname,QType(QType::A), res,depth+1,beenthere) && !res.empty()) 
-    ret=res[res.size()-1].content; // last entry, in case of CNAME in between
+  vector<string> ret;
 
+  if(!doResolve(qname,QType(QType::A), res,depth+1,beenthere) && !res.empty()) {
+    for(res_t::const_iterator i=res.begin(); i!= res.end(); ++i)
+      if(i->qtype.getCode()==QType::A)
+	ret.push_back(i->content);
+  }
+  if(ret.size() > 1)
+    random_shuffle(ret.begin(), ret.end());
   return ret;
 }
 
@@ -128,12 +136,15 @@ void SyncRes::getBestNSFromCache(const string &qname, set<DNSResourceRecord>&bes
 	  rr.content=toLowerCanonic(k->content);
 	  if(!endsOn(rr.content,subdomain) || RC.get(d_now.tv_sec, rr.content ,QType(QType::A),&aset) > 5) {
 	    bestns.insert(rr);
+	    
 	    LOG<<prefix<<qname<<": NS (with ip, or non-glue) in cache for '"<<subdomain<<"' -> '"<<rr.content<<"'"<<endl;
 	    LOG<<prefix<<qname<<": within bailiwick: "<<endsOn(rr.content,subdomain);
-	    if(!aset.empty())
-	      L<<", in cache, ttl="<<(unsigned int)(((time_t)aset.begin()->ttl- d_now.tv_sec ))<<endl;
-	    else
-	      L<<", not in cache"<<endl;
+	    if(!aset.empty()) {
+	      LOG<<", in cache, ttl="<<(unsigned int)(((time_t)aset.begin()->ttl- d_now.tv_sec ))<<endl;
+	    }
+	    else {
+	      LOG<<", not in cache"<<endl;
+	    }
 	  }
 	  else
 	    LOG<<prefix<<qname<<": NS in cache for '"<<subdomain<<"', but needs glue ("<<toLowerCanonic(k->content)<<") which we miss or is expired"<<endl;
@@ -221,7 +232,7 @@ bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSRe
 
   if(s_negcache.count(toLower(qname))) {
     res=0;
-    map<string,NegCacheEntry>::const_iterator ni=s_negcache.find(toLower(qname));
+    negcache_t::const_iterator ni=s_negcache.find(toLower(qname));
     if(d_now.tv_sec < ni->second.ttd) {
       sttl=ni->second.ttd - d_now.tv_sec;
       LOG<<prefix<<qname<<": Entire record '"<<toLower(qname)<<"', is negatively cached for another "<<sttl<<" seconds"<<endl;
@@ -229,7 +240,6 @@ bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSRe
       giveNegative=true;
       sqname=ni->second.name;
       sqt="SOA";
-
     }
     else {
       LOG<<prefix<<qname<<": Entire record '"<<toLower(qname)<<"' was negatively cached, but entry expired"<<endl;
@@ -238,11 +248,11 @@ bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSRe
   }
 
   if(!giveNegative) { // let's try some more
-    tuple=toLower(qname)+"|"+qtype.getName();
+    tuple=toLower(qname); tuple.append(1,'|'); tuple+=qtype.getName();
     LOG<<prefix<<qname<<": Looking for direct cache hit of '"<<tuple<<"', negative cached: "<<s_negcache.count(tuple)<<endl;
 
     res=0;
-    map<string,NegCacheEntry>::const_iterator ni=s_negcache.find(tuple);
+    negcache_t::const_iterator ni=s_negcache.find(tuple);
     if(ni!=s_negcache.end()) {
       if(d_now.tv_sec < ni->second.ttd) {
 	sttl=ni->second.ttd - d_now.tv_sec;
@@ -304,7 +314,7 @@ bool SyncRes::moreSpecificThan(const string& a, const string &b)
   return counta>countb;
 }
 
-static map<string,DecayingEwma> nsSpeeds;
+
 
 struct speedOrder
 {
@@ -324,7 +334,7 @@ inline vector<string> SyncRes::shuffle(set<string> &nameservers, const string &p
 
   for(set<string>::const_iterator i=nameservers.begin();i!=nameservers.end();++i) {
     rnameservers.push_back(*i);
-    DecayingEwma& temp=nsSpeeds[toLower(*i)];
+    DecayingEwma& temp=s_nsSpeeds[toLower(*i)];
     speeds[*i]=temp.get(&d_now);
   }
   random_shuffle(rnameservers.begin(),rnameservers.end());
@@ -371,48 +381,55 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	continue;
       }
       LOG<<prefix<<qname<<": Trying to resolve NS "<<*tns<<" ("<<1+tns-rnameservers.begin()<<"/"<<rnameservers.size()<<")"<<endl;
-      string remoteIP=getA(*tns, depth+1,beenthere);
-      if(remoteIP.empty()) {
+      typedef vector<string> remoteIPs_t;
+      remoteIPs_t remoteIPs=getAs(*tns, depth+1, beenthere);
+      if(remoteIPs.empty()) {
 	LOG<<prefix<<qname<<": Failed to get IP for NS "<<*tns<<", trying next if available"<<endl;
 	continue;
       }
-      LOG<<prefix<<qname<<": Resolved '"+auth+"' NS "<<*tns<<" to "<<remoteIP<<", asking '"<<qname<<"|"<<qtype.getName()<<"'"<<endl;
-
+      remoteIPs_t::const_iterator remoteIP;
       bool doTCP=false;
-
-      if(s_throttle.shouldThrottle(d_now.tv_sec, remoteIP+"|"+qname+"|"+qtype.getName())) {
-	LOG<<prefix<<qname<<": query throttled "<<endl;
-	s_throttledqueries++;
-	d_throttledqueries++;
-	continue;
-      }
-      else {
-	s_outqueries++;
-	d_outqueries++;
-      TryTCP:
-	if(doTCP) {
-	  s_tcpoutqueries++;
-	  d_tcpoutqueries++;
-	}
-
-	int ret=d_lwr.asyncresolve(remoteIP, qname.c_str(), qtype.getCode(), doTCP);    // <- we go out on the wire!
-	if(ret != 1) {
-	  if(ret==0) {
-	    LOG<<prefix<<qname<<": timeout resolving"<<endl;
-	    d_timeouts++;
-	    s_outgoingtimeouts++;
-	  }
-	  else
-	    LOG<<prefix<<qname<<": error resolving"<<endl;
-
-	  nsSpeeds[toLower(*tns)].submit(1000000, &d_now); // 1 sec
-	  
-	  s_throttle.throttle(d_now.tv_sec, remoteIP+"|"+qname+"|"+qtype.getName(),20,5);
+      for(remoteIP = remoteIPs.begin(); remoteIP != remoteIPs.end(); ++remoteIP) {
+	LOG<<prefix<<qname<<": Resolved '"+auth+"' NS "<<*tns<<" to "<<*remoteIP<<", asking '"<<qname<<"|"<<qtype.getName()<<"'"<<endl;
+	
+	if(s_throttle.shouldThrottle(d_now.tv_sec, *remoteIP+"|"+qname+"|"+qtype.getName())) {
+	  LOG<<prefix<<qname<<": query throttled "<<endl;
+	  s_throttledqueries++;
+	  d_throttledqueries++;
 	  continue;
 	}
-	gettimeofday(&d_now, 0);
+	else {
+	  s_outqueries++;
+	  d_outqueries++;
+	TryTCP:
+	  if(doTCP) {
+	    s_tcpoutqueries++;
+	    d_tcpoutqueries++;
+	  }
+	  
+	  int ret=d_lwr.asyncresolve(*remoteIP, qname.c_str(), qtype.getCode(), doTCP);    // <- we go out on the wire!
+	  if(ret != 1) {
+	    if(ret==0) {
+	      LOG<<prefix<<qname<<": timeout resolving"<<endl;
+	      d_timeouts++;
+	      s_outgoingtimeouts++;
+	    }
+	    else
+	      LOG<<prefix<<qname<<": error resolving"<<endl;
+	    
+	    s_nsSpeeds[toLower(*tns)].submit(1000000, &d_now); // 1 sec
+	  
+	    s_throttle.throttle(d_now.tv_sec, *remoteIP+"|"+qname+"|"+qtype.getName(),20,5);
+	    continue;
+	  }
+	  gettimeofday(&d_now, 0);
+	  break; // it did work!
+	}
       }
 
+      if(remoteIP == remoteIPs.end())  // we tried all IP addresses, none worked
+	continue; 
+
       result=d_lwr.result();
       
       if(d_lwr.d_tcbit) {
@@ -427,11 +444,11 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 
       if(d_lwr.d_rcode==RCode::ServFail) {
 	LOG<<prefix<<qname<<": "<<*tns<<" returned a ServFail, trying sibling NS"<<endl;
-	s_throttle.throttle(d_now.tv_sec,remoteIP+"|"+qname+"|"+qtype.getName(),60,3);
+	s_throttle.throttle(d_now.tv_sec,*remoteIP+"|"+qname+"|"+qtype.getName(),60,3);
 	continue;
       }
-      LOG<<prefix<<qname<<": Got "<<result.size()<<" answers from "<<*tns<<" ("<<remoteIP<<"), rcode="<<d_lwr.d_rcode<<", in "<<d_lwr.d_usec/1000<<"ms"<<endl;
-      nsSpeeds[toLower(*tns)].submit(d_lwr.d_usec, &d_now);
+      LOG<<prefix<<qname<<": Got "<<result.size()<<" answers from "<<*tns<<" ("<<*remoteIP<<"), rcode="<<d_lwr.d_rcode<<", in "<<d_lwr.d_usec/1000<<"ms"<<endl;
+      s_nsSpeeds[toLower(*tns)].submit(d_lwr.d_usec, &d_now);
 
       map<string,set<DNSResourceRecord> > tcache;
       // reap all answers from this packet that are acceptable
@@ -444,7 +461,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	}
 	
 	if(endsOn(i->qname, auth)) {
-	  if(d_lwr.d_aabit && d_lwr.d_rcode==RCode::NoError && i->d_place==DNSResourceRecord::ANSWER && arg().contains("delegation-only",auth)) {
+	  if(d_lwr.d_aabit && d_lwr.d_rcode==RCode::NoError && i->d_place==DNSResourceRecord::ANSWER && ::arg().contains("delegation-only",auth)) {
 	    LOG<<"NO! Is from delegation-only zone"<<endl;
 	    s_nodelegated++;
 	    return RCode::NXDomain;
@@ -561,7 +578,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
       }
       else {
 	LOG<<prefix<<qname<<": status=NS "<<*tns<<" is lame for '"<<auth<<"', trying sibling NS"<<endl;
-	s_throttle.throttle(d_now.tv_sec, remoteIP+"|"+qname+"|"+qtype.getName(),60,0);
+	s_throttle.throttle(d_now.tv_sec, *remoteIP+"|"+qname+"|"+qtype.getName(),60,0);
       }
     }
   }
@@ -580,6 +597,7 @@ void SyncRes::addCruft(const string &qname, vector<DNSResourceRecord>& ret)
 
   LOG<<d_prefix<<qname<<": Starting additional processing"<<endl;
   vector<DNSResourceRecord> addit;
+  bool doIPv6AP=::arg().mustDo("aaaa-additional-processing");
   for(vector<DNSResourceRecord>::const_iterator k=ret.begin();k!=ret.end();++k) 
     if((k->d_place==DNSResourceRecord::ANSWER && k->qtype==QType(QType::MX)) || 
        ((k->d_place==DNSResourceRecord::AUTHORITY || k->d_place==DNSResourceRecord::ANSWER) && k->qtype==QType(QType::NS))) {
@@ -587,16 +605,22 @@ void SyncRes::addCruft(const string &qname, vector<DNSResourceRecord>& ret)
       set<GetBestNSAnswer>beenthere;
       if(k->qtype==QType(QType::MX)) {
 	string::size_type pos=k->content.find_first_not_of(" \t0123456789"); // chop off the priority
-	if(pos!=string::npos)
+	if(pos!=string::npos) {
 	  doResolve(toLowerCanonic(k->content.substr(pos)), QType(QType::A),addit,1,beenthere);
-	else
+	  if(doIPv6AP)
+	    doResolve(toLowerCanonic(k->content.substr(pos)), QType(QType::AAAA),addit,1,beenthere);
+	}
+	else {
 	  doResolve(toLowerCanonic(k->content), QType(QType::A),addit,1,beenthere);
+	  if(doIPv6AP)
+	    doResolve(toLowerCanonic(k->content.substr(pos)), QType(QType::AAAA),addit,1,beenthere);
+	}
       }
-      else
+      else {
 	doResolve(k->content,QType(QType::A),addit,1,beenthere);
-
-      if(arg().mustDo("aaaa-additional-processing"))
-	doResolve(k->content,QType(QType::AAAA),addit,1,beenthere);
+	if(doIPv6AP)
+	  doResolve(k->content,QType(QType::AAAA),addit,1,beenthere);
+      }
     }
   
   for(vector<DNSResourceRecord>::iterator k=addit.begin();k!=addit.end();++k) {
