@@ -162,7 +162,7 @@ void *TCPNameserver::doConnection(void *data)
       DNSPacket *packet=new DNSPacket;
 
       packet->setRemote((struct sockaddr *)&remote,sizeof(remote));
-
+      packet->d_tcp=true;
       if(packet->parse(mesg, pktlen)<0)
 	break;
       
@@ -301,6 +301,7 @@ int TCPNameserver::doAXFR(const string &target, DNSPacket *q, int outsock)
     DLOG(L<<"Looking for SOA"<<endl);
 
     if(!s_P->getBackend()->getSOA(target,sd)) {
+      L<<Logger::Error<<"AXFR of domain '"<<target<<"' failed: not authoritative"<<endl;
       outpacket->setRcode(9); // 'NOTAUTH'
       sendDelPacket(outpacket,outsock);
       return 0;
@@ -311,6 +312,7 @@ int TCPNameserver::doAXFR(const string &target, DNSPacket *q, int outsock)
 
   sd.db=(DNSBackend *)-1; // force uncached answer
   if(!P.getBackend()->getSOA(target,sd)) {
+      L<<Logger::Error<<"AXFR of domain '"<<target<<"' failed: not authoritative in second instance"<<endl;
     outpacket->setRcode(9); // 'NOTAUTH'
     sendDelPacket(outpacket,outsock);
     return 0;
@@ -509,7 +511,7 @@ TCPNameserver::TCPNameserver()
     }
     
     listen(s,128);
-    L<<Logger::Error<<"TCPv6 server bound to "<<*laddr<<":"<<arg()["local-port"]<<endl;
+    L<<Logger::Error<<"TCPv6 server bound to ["<<*laddr<<"]:"<<arg()["local-port"]<<endl;
     d_sockets.push_back(s);
     FD_SET(s, &d_rfds);
     d_highfd=max(s,d_highfd);
@@ -532,7 +534,10 @@ void TCPNameserver::thread()
 
       fd_set rfds=d_rfds; 
 
-      select(d_highfd+1, &rfds, 0, 0,  0); // blocks
+      int ret=select(d_highfd+1, &rfds, 0, 0,  0); // blocks
+      if(ret <= 0)
+	continue;
+
       int sock=-1;
       for(vector<int>::const_iterator i=d_sockets.begin();i!=d_sockets.end();++i) {
 	if(FD_ISSET(*i, &rfds)) {
